@@ -37,6 +37,7 @@
 #include "../common/sysutils.h" /* (gnupg_fd_t) */
 #include "../common/session-env.h"
 #include "../common/shareddefs.h"
+#include "../common/name-value.h"
 
 /* To convey some special hash algorithms we use algorithm numbers
    reserved for application use. */
@@ -157,6 +158,13 @@ struct
      interactively mark certificate in trustlist.txt as trusted. */
   int allow_mark_trusted;
 
+  /* Only use the system trustlist.  */
+  int no_user_trustlist;
+
+  /* The standard system trustlist is SYSCONFDIR/trustlist.txt.  This
+   * option can be used to change the name.  */
+  const char *sys_trustlist_name;
+
   /* If this global option is true, the Assuan command
      PRESET_PASSPHRASE is allowed.  */
   int allow_preset_passphrase;
@@ -392,6 +400,7 @@ struct card_key_info_s
   char keygrip[41];
   char *serialno;
   char *idstr;
+  char *usage;
 };
 
 /*-- gpg-agent.c --*/
@@ -461,12 +470,15 @@ gpg_error_t agent_key_from_file (ctrl_t ctrl,
                                  cache_mode_t cache_mode,
                                  lookup_ttl_t lookup_ttl,
                                  gcry_sexp_t *result,
-                                 char **r_passphrase);
+                                 char **r_passphrase, time_t *r_timestamp);
 gpg_error_t agent_raw_key_from_file (ctrl_t ctrl, const unsigned char *grip,
-                                     gcry_sexp_t *result);
+                                     gcry_sexp_t *result, nvc_t *r_keymeta);
 gpg_error_t agent_public_key_from_file (ctrl_t ctrl,
                                         const unsigned char *grip,
                                         gcry_sexp_t *result);
+gpg_error_t agent_ssh_key_from_file (ctrl_t ctrl,
+                                     const unsigned char *grip,
+                                     gcry_sexp_t *result);
 int agent_pk_get_algo (gcry_sexp_t s_key);
 int agent_is_tpm2_key(gcry_sexp_t s_key);
 int agent_key_available (const unsigned char *grip);
@@ -477,6 +489,7 @@ gpg_error_t agent_key_info_from_file (ctrl_t ctrl, const unsigned char *grip,
 gpg_error_t agent_delete_key (ctrl_t ctrl, const char *desc_text,
                               const unsigned char *grip,
                               int force, int only_stubs);
+gpg_error_t agent_update_private_key (const unsigned char *grip, nvc_t pk);
 
 /*-- call-pinentry.c --*/
 void initialize_module_call_pinentry (void);
@@ -593,11 +606,11 @@ void agent_reload_trustlist (void);
 
 /*-- divert-tpm2.c --*/
 #ifdef HAVE_LIBTSS
-int divert_tpm2_pksign (ctrl_t ctrl, const char *desc_text,
+int divert_tpm2_pksign (ctrl_t ctrl,
                         const unsigned char *digest, size_t digestlen, int algo,
                         const unsigned char *shadow_info, unsigned char **r_sig,
                         size_t *r_siglen);
-int divert_tpm2_pkdecrypt (ctrl_t ctrl, const char *desc_text,
+int divert_tpm2_pkdecrypt (ctrl_t ctrl,
                            const unsigned char *cipher,
                            const unsigned char *shadow_info,
                            char **r_buf, size_t *r_len, int *r_padding);
@@ -605,25 +618,25 @@ int divert_tpm2_writekey (ctrl_t ctrl, const unsigned char *grip,
                           gcry_sexp_t s_skey);
 #else /*!HAVE_LIBTSS*/
 static inline int
-divert_tpm2_pksign (ctrl_t ctrl, const char *desc_text,
+divert_tpm2_pksign (ctrl_t ctrl,
                     const unsigned char *digest,
                     size_t digestlen, int algo,
                     const unsigned char *shadow_info,
                     unsigned char **r_sig,
                     size_t *r_siglen)
 {
-  (void)ctrl; (void)desc_text; (void)digest; (void)digestlen;
+  (void)ctrl; (void)digest; (void)digestlen;
   (void)algo; (void)shadow_info; (void)r_sig; (void)r_siglen;
   return gpg_error (GPG_ERR_NOT_SUPPORTED);
 }
 static inline int
-divert_tpm2_pkdecrypt (ctrl_t ctrl, const char *desc_text,
+divert_tpm2_pkdecrypt (ctrl_t ctrl,
                        const unsigned char *cipher,
                        const unsigned char *shadow_info,
                        char **r_buf, size_t *r_len,
                        int *r_padding)
 {
-  (void)ctrl; (void)desc_text; (void)cipher; (void)shadow_info;
+  (void)ctrl; (void)cipher; (void)shadow_info;
   (void)r_buf; (void)r_len; (void)r_padding;
   return gpg_error (GPG_ERR_NOT_SUPPORTED);
 }
@@ -639,15 +652,14 @@ divert_tpm2_writekey (ctrl_t ctrl, const unsigned char *grip,
 
 
 /*-- divert-scd.c --*/
-int divert_pksign (ctrl_t ctrl, const char *desc_text,
+int divert_pksign (ctrl_t ctrl,
                    const unsigned char *grip,
                    const unsigned char *digest, size_t digestlen, int algo,
-                   const unsigned char *shadow_info, unsigned char **r_sig,
+                   unsigned char **r_sig,
                    size_t *r_siglen);
-int divert_pkdecrypt (ctrl_t ctrl, const char *desc_text,
+int divert_pkdecrypt (ctrl_t ctrl,
                       const unsigned char *grip,
                       const unsigned char *cipher,
-                      const unsigned char *shadow_info,
                       char **r_buf, size_t *r_len, int *r_padding);
 int divert_generic_cmd (ctrl_t ctrl,
                         const char *cmdline, void *assuan_context);
