@@ -1,6 +1,6 @@
 /* exechelp.c - Fork and exec helpers for POSIX
- * Copyright (C) 2004, 2007, 2008, 2009,
- *               2010 Free Software Foundation, Inc.
+ * Copyright (C) 2004, 2007-2009, 2010 Free Software Foundation, Inc.
+ * Copyright (C) 2004, 2006-2012, 2014-2017 g10 Code GmbH
  *
  * This file is part of GnuPG.
  *
@@ -26,6 +26,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * SPDX-License-Identifier: (LGPL-3.0+ OR GPL-2.0+)
  */
 
 #include <config.h>
@@ -276,16 +277,21 @@ get_all_open_fds (void)
 static void
 do_exec (const char *pgmname, const char *argv[],
          int fd_in, int fd_out, int fd_err,
-         int *except, void (*preexec)(void) )
+         int *except, void (*preexec)(void), unsigned int flags)
 {
   char **arg_list;
   int i, j;
   int fds[3];
+  int nodevnull[3];
 
   fds[0] = fd_in;
   fds[1] = fd_out;
   fds[2] = fd_err;
 
+  nodevnull[0] = !!(flags & GNUPG_SPAWN_KEEP_STDIN);
+  nodevnull[1] = !!(flags & GNUPG_SPAWN_KEEP_STDOUT);
+  nodevnull[2] = !!(flags & GNUPG_SPAWN_KEEP_STDERR);
+
   /* Create the command line argument array.  */
   i = 0;
   if (argv)
@@ -304,7 +310,9 @@ do_exec (const char *pgmname, const char *argv[],
   /* Assign /dev/null to unused FDs. */
   for (i=0; i <= 2; i++)
     {
-      if (fds[i] == -1 )
+      if (nodevnull[i])
+        continue;
+      if (fds[i] == -1)
         {
           fds[i] = open ("/dev/null", i? O_WRONLY : O_RDONLY);
           if (fds[i] == -1)
@@ -316,6 +324,8 @@ do_exec (const char *pgmname, const char *argv[],
   /* Connect the standard files.  */
   for (i=0; i <= 2; i++)
     {
+      if (nodevnull[i])
+        continue;
       if (fds[i] != i && dup2 (fds[i], i) == -1)
         log_fatal ("dup2 std%s failed: %s\n",
                    i==0?"in":i==1?"out":"err", strerror (errno));
@@ -415,6 +425,15 @@ gnupg_create_pipe (int filedes[2])
 }
 
 
+/* Close the end of a pipe.  */
+void
+gnupg_close_pipe (int fd)
+{
+  if (fd != -1)
+    close (fd);
+}
+
+
 /* Fork and exec the PGMNAME, see exechelp.h for details.  */
 gpg_error_t
 gnupg_spawn_process (const char *pgmname, const char *argv[],
@@ -525,7 +544,7 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
       es_fclose (outfp);
       es_fclose (errfp);
       do_exec (pgmname, argv, inpipe[0], outpipe[1], errpipe[1],
-               except, preexec);
+               except, preexec, flags);
       /*NOTREACHED*/
     }
 
@@ -575,7 +594,7 @@ gnupg_spawn_process_fd (const char *pgmname, const char *argv[],
     {
       gcry_control (GCRYCTL_TERM_SECMEM);
       /* Run child. */
-      do_exec (pgmname, argv, infd, outfd, errfd, NULL, NULL);
+      do_exec (pgmname, argv, infd, outfd, errfd, NULL, NULL, 0);
       /*NOTREACHED*/
     }
 
@@ -722,8 +741,13 @@ gnupg_wait_processes (const char **pgmnames, pid_t *pids, size_t count,
     {
       int status = -1;
 
+      /* Skip invalid PID.  */
       if (pids[i] == (pid_t)(-1))
-        return my_error (GPG_ERR_INV_VALUE);
+        {
+          r_exitcodes[i] = -1;
+          left -= 1;
+          continue;
+        }
 
       /* See if there was a previously stored result for this pid.  */
       if (get_result (pids[i], &status))
@@ -869,7 +893,7 @@ gnupg_spawn_process_detached (const char *pgmname, const char *argv[],
         for (i=0; envp[i]; i++)
           putenv (xstrdup (envp[i]));
 
-      do_exec (pgmname, argv, -1, -1, -1, NULL, NULL);
+      do_exec (pgmname, argv, -1, -1, -1, NULL, NULL, 0);
 
       /*NOTREACHED*/
     }
