@@ -65,6 +65,8 @@
 #include "sysutils.h"
 #include "exechelp.h"
 
+#include <windows.h>
+
 /* Define to 1 do enable debugging.  */
 #define DEBUG_W32_SPAWN 0
 
@@ -75,12 +77,11 @@
 #undef X_OK
 #define X_OK F_OK
 
-/* We assume that a HANDLE can be represented by an int which should
-   be true for all i386 systems (HANDLE is defined as void *) and
-   these are the only systems for which Windows is available.  Further
-   we assume that -1 denotes an invalid handle.  */
+/* We assume that a HANDLE can be represented by an intptr_t which
+   should be true for all systems (HANDLE is defined as void *).
+   Further we assume that -1 denotes an invalid handle.  */
 # define fd_to_handle(a)  ((HANDLE)(a))
-# define handle_to_fd(a)  ((int)(a))
+# define handle_to_fd(a)  ((intptr_t)(a))
 # define pid_to_handle(a) ((HANDLE)(a))
 # define handle_to_pid(a) ((int)(a))
 
@@ -554,11 +555,14 @@ gnupg_spawn_process (const char *pgmname, const char *argv[],
     return err;
 
   if (inpipe[0] == INVALID_HANDLE_VALUE)
-    nullhd[0] = w32_open_null (0);
+    nullhd[0] = ((flags & GNUPG_SPAWN_KEEP_STDIN)?
+                 GetStdHandle (STD_INPUT_HANDLE) : w32_open_null (0));
   if (outpipe[1] == INVALID_HANDLE_VALUE)
-    nullhd[1] = w32_open_null (1);
+    nullhd[1] = ((flags & GNUPG_SPAWN_KEEP_STDOUT)?
+                 GetStdHandle (STD_OUTPUT_HANDLE) : w32_open_null (1));
   if (errpipe[1] == INVALID_HANDLE_VALUE)
-    nullhd[2] = w32_open_null (1);
+    nullhd[2] = ((flags & GNUPG_SPAWN_KEEP_STDOUT)?
+                 GetStdHandle (STD_ERROR_HANDLE) : w32_open_null (1));
 
   /* Start the process.  Note that we can't run the PREEXEC function
      because this might change our own environment. */
@@ -803,7 +807,7 @@ gnupg_wait_processes (const char **pgmnames, pid_t *pids, size_t count,
       if (pids[i] == (pid_t)(-1))
         return my_error (GPG_ERR_INV_VALUE);
 
-      procs[i] = fd_to_handle (pids[i]);
+      procs[i] = pid_to_handle (pids[i]);
     }
 
   /* FIXME: We should do a pth_waitpid here.  However this has not yet
@@ -903,10 +907,14 @@ gnupg_spawn_process_detached (const char *pgmname, const char *argv[],
   BOOL in_job = FALSE;
   gpg_err_code_t ec;
   int rc;
+  int jobdebug;
 
   /* We don't use ENVP.  */
   (void)envp;
 
+  cmdline = getenv ("GNUPG_EXEC_DEBUG_FLAGS");
+  jobdebug = (cmdline && (atoi (cmdline) & 1));
+
   if ((ec = gnupg_access (pgmname, X_OK)))
     return gpg_err_make (default_errsource, ec);
 
@@ -955,24 +963,32 @@ gnupg_spawn_process_detached (const char *pgmname, const char *argv[],
       else if ((info.BasicLimitInformation.LimitFlags &
                 JOB_OBJECT_LIMIT_BREAKAWAY_OK))
         {
-          log_info ("Using CREATE_BREAKAWAY_FROM_JOB flag\n");
+          if (jobdebug)
+            log_debug ("Using CREATE_BREAKAWAY_FROM_JOB flag\n");
           cr_flags |= CREATE_BREAKAWAY_FROM_JOB;
         }
       else if ((info.BasicLimitInformation.LimitFlags &
                 JOB_OBJECT_LIMIT_SILENT_BREAKAWAY_OK))
         {
           /* The child process should automatically detach from the job. */
-          log_info ("Not using CREATE_BREAKAWAY_FROM_JOB flag; "
-                     "JOB_OBJECT_LIMIT_SILENT_BREAKAWAY_OK is set\n");
+          if (jobdebug)
+            log_debug ("Not using CREATE_BREAKAWAY_FROM_JOB flag; "
+                       "JOB_OBJECT_LIMIT_SILENT_BREAKAWAY_OK is set\n");
         }
       else
         {
           /* It seems that the child process must remain in the job.
            * This is not necessarily an error, although it can cause premature
            * termination of the child process when the job is closed. */
-          log_info ("Not using CREATE_BREAKAWAY_FROM_JOB flag\n");
+          if (jobdebug)
+            log_debug ("Not using CREATE_BREAKAWAY_FROM_JOB flag\n");
         }
     }
+  else
+    {
+      if (jobdebug)
+        log_debug ("Process is not in a Job\n");
+    }
 
   /*   log_debug ("CreateProcess(detached), path='%s' cmdline='%s'\n", */
   /*              pgmname, cmdline); */
