@@ -53,7 +53,6 @@ enum
 
     STATUS_NEED_PASSPHRASE,
     STATUS_VALIDSIG,
-    STATUS_ASSERT_SIGNER,
     STATUS_SIG_ID,
     STATUS_ENC_TO,
     STATUS_NODATA,
@@ -152,7 +151,6 @@ enum
     STATUS_TRUNCATED,
     STATUS_MOUNTPOINT,
     STATUS_BLOCKDEV,
-    STATUS_PLAINDEV,           /* The decrypted virtual device.  */
 
     STATUS_PINENTRY_LAUNCHED,
 
