@@ -783,11 +783,6 @@ w32_strerror (int ec)
 
   if (ec == -1)
     ec = (int)GetLastError ();
-#ifdef HAVE_W32CE_SYSTEM
-  /* There is only a wchar_t FormatMessage.  It does not make much
-     sense to play the conversion game; we print only the code.  */
-  snprintf (strerr, sizeof strerr, "ec=%d", (int)GetLastError ());
-#else
   FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, ec,
                  MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
                  strerr, DIM (strerr)-1, NULL);
@@ -797,7 +792,6 @@ w32_strerror (int ec)
     if (n > 2 && strerr[n-2] == '\r' && strerr[n-1] == '\n' )
       strerr[n-2] = 0;
   }
-#endif
   return strerr;
 }
 #endif /*HAVE_W32_SYSTEM*/
@@ -1169,9 +1163,10 @@ try_percent_escape (const char *str, const char *extra)
 }
 
 
-
-static char *
-do_strconcat (const char *s1, va_list arg_ptr)
+/* Same as strconcat but takes a va_list.  Returns EINVAL if the list
+ * is too long, all other errors are due to an ENOMEM condition.  */
+char *
+vstrconcat (const char *s1, va_list arg_ptr)
 {
   const char *argv[48];
   size_t argc;
@@ -1216,7 +1211,7 @@ strconcat (const char *s1, ...)
   else
     {
       va_start (arg_ptr, s1);
-      result = do_strconcat (s1, arg_ptr);
+      result = vstrconcat (s1, arg_ptr);
       va_end (arg_ptr);
     }
   return result;
@@ -1235,7 +1230,7 @@ xstrconcat (const char *s1, ...)
   else
     {
       va_start (arg_ptr, s1);
-      result = do_strconcat (s1, arg_ptr);
+      result = vstrconcat (s1, arg_ptr);
       va_end (arg_ptr);
     }
   if (!result)
@@ -1692,3 +1687,107 @@ format_text (const char *text_in, int target_cols, int max_cols)
 
   return text;
 }
+
+
+/* Substitute environment variables in STRING and return a new string.
+ * On error the function returns NULL.  */
+char *
+substitute_envvars (const char *string)
+{
+  char *line, *p, *pend;
+  const char *value;
+  size_t valuelen, n;
+  char *result = NULL;
+
+  result = line = xtrystrdup (string);
+  if (!result)
+    return NULL; /* Ooops */
+
+  while (*line)
+    {
+      p = strchr (line, '$');
+      if (!p)
+        goto leave; /* No or no more variables.  */
+
+      if (p[1] == '$') /* Escaped dollar sign. */
+        {
+          memmove (p, p+1, strlen (p+1)+1);
+          line = p + 1;
+          continue;
+        }
+
+      if (p[1] == '{')
+        {
+          int count = 0;
+
+          for (pend=p+2; *pend; pend++)
+            {
+              if (*pend == '{')
+                count++;
+              else if (*pend == '}')
+                {
+                  if (--count < 0)
+                    break;
+                }
+            }
+          if (!*pend)
+            goto leave; /* Unclosed - don't substitute.  */
+        }
+      else
+        {
+          for (pend = p+1; *pend && (alnump (pend) || *pend == '_'); pend++)
+            ;
+        }
+
+      if (p[1] == '{' && *pend == '}')
+        {
+          int save = *pend;
+          *pend = 0;
+          value = getenv (p+2);
+          *pend++ = save;
+        }
+      else
+        {
+          int save = *pend;
+          *pend = 0;
+          value = getenv (p+1);
+          *pend = save;
+        }
+
+      if (!value)
+        value = "";
+      valuelen = strlen (value);
+      if (valuelen <= pend - p)
+        {
+          memcpy (p, value, valuelen);
+          p += valuelen;
+          n = pend - p;
+          if (n)
+            memmove (p, p+n, strlen (p+n)+1);
+          line = p;
+        }
+      else
+        {
+          char *src = result;
+          char *dst;
+
+          dst = xtrymalloc (strlen (src) + valuelen + 1);
+          if (!dst)
+            {
+              xfree (result);
+              return NULL;
+            }
+          n = p - src;
+          memcpy (dst, src, n);
+          memcpy (dst + n, value, valuelen);
+          n += valuelen;
+          strcpy (dst + n, pend);
+          line = dst + n;
+          xfree (result);
+          result = dst;
+        }
+    }
+
+ leave:
+  return result;
+}
