@@ -55,6 +55,10 @@
 #include "stringhelp.h"
 #include "utf8conv.h"
 
+#ifdef HAVE_W32_SYSTEM
+#include <windows.h>
+#endif
+
 #ifndef MB_LEN_MAX
 #define MB_LEN_MAX 16
 #endif
@@ -138,7 +142,7 @@ handle_iconv_error (const char *to, const char *from, int use_fallback)
          native encoding.  Nowadays this seems to be the best bet in
          case of errors from iconv or nl_langinfo.  */
       active_charset_name = "utf-8";
-      no_translation = 0;
+      no_translation = 1;
       use_iconv = 0;
     }
 }
@@ -166,10 +170,8 @@ set_native_charset (const char *newset)
          different one for console input.  Not sure how to cope with
          that.  If the console Code page is not known we fall back to
          the system code page.  */
-#ifndef HAVE_W32CE_SYSTEM
       cpno = GetConsoleOutputCP ();
       if (!cpno)
-#endif
         cpno = GetACP ();
       sprintf (codepage, "CP%u", cpno );
       /* Resolve alias.  We use a long string string and not the usual
@@ -802,10 +804,8 @@ get_w32_codepage (void)
 
   if (!cp)
     {
-#ifndef HAVE_W32CE_SYSTEM
       cp = GetConsoleOutputCP ();
       if (!cp)
-#endif
         cp = GetACP ();
     }
   return cp;
