@@ -21,6 +21,7 @@
 
 #include <config.h>
 
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -48,7 +49,6 @@
 
 #include <gpg-error.h>
 #include "../common/logging.h"
-#include "../common/argparse.h"
 #include "../common/stringhelp.h"
 #include "../common/mischelp.h"
 #include "../common/strlist.h"
@@ -92,14 +92,13 @@ enum
     oStartTLS,
     oLdapTLS,
     oNtds,
-
     oOnlySearchTimeout,
     oLogWithPID
   };
 
 
 /* The list of options as used by the argparse.c code.  */
-static ARGPARSE_OPTS opts[] = {
+static gpgrt_opt_t opts[] = {
   { oVerbose,  "verbose",   0, "verbose" },
   { oQuiet,    "quiet",     0, "be somewhat more quiet" },
   { oTimeout,  "timeout",   1, "|N|set LDAP timeout to N seconds"},
@@ -165,7 +164,7 @@ my_strusage (int level)
 {
   const char *p;
 
-  switch(level)
+  switch (level)
     {
     case  9: p = "GPL-3.0-or-later"; break;
     case 11: p = "dirmngr_ldap (@GNUPG@)";
@@ -194,7 +193,7 @@ my_strusage (int level)
 int
 main (int argc, char **argv)
 {
-  ARGPARSE_ARGS pargs;
+  gpgrt_argparse_t pargs;
   int any_err = 0;
   char *p;
   int only_search_timeout = 0;
@@ -203,7 +202,7 @@ main (int argc, char **argv)
 
   early_system_init ();
 
-  set_strusage (my_strusage);
+  gpgrt_set_strusage (my_strusage);
   log_set_prefix ("dirmngr_ldap", GPGRT_LOG_WITH_PREFIX);
 
   init_common_subsystems (&argc, &argv);
@@ -220,7 +219,7 @@ main (int argc, char **argv)
   pargs.argc = &argc;
   pargs.argv = &argv;
   pargs.flags= ARGPARSE_FLAG_KEEP;
-  while (gnupg_argparse (NULL, &pargs, opts))
+  while (gpgrt_argparse (NULL, &pargs, opts))
     {
       switch (pargs.r_opt)
         {
@@ -259,7 +258,7 @@ main (int argc, char **argv)
           break;
 	}
     }
-  gnupg_argparse (NULL, &pargs, NULL);  /* Release internal state.  */
+  gpgrt_argparse (NULL, &pargs, NULL);
 
   if (only_search_timeout)
     opt.alarm_timeout = 0;
@@ -382,7 +381,7 @@ set_timeout (void)
           sec_attr.nLength = sizeof sec_attr;
           sec_attr.bInheritHandle = FALSE;
 
-          /* Create a manual resetable timer.  */
+          /* Create a manual resettable timer.  */
           timer = CreateWaitableTimer (NULL, TRUE, NULL);
           /* Initially set the timer.  */
           SetWaitableTimer (timer, &due_time, 0, NULL, NULL, 0);
@@ -747,7 +746,7 @@ print_ldap_entries (LDAP *ld, LDAPMessage *msg, char *want_attr)
 
 /* Fetch data from the server at LD using FILTER.  */
 static int
-fetch_ldap (LDAP *ld, char *base, int scope, char *filter)
+fetch_ldap (LDAP *ld, const char *base, int scope, const char *filter)
 {
   gpg_error_t err;
   int lerr;
