@@ -26,18 +26,6 @@
 void ldapserver_list_free (ldap_server_t servers);
 
 
-/* Parse a single LDAP server configuration line.  Returns the server
-   or NULL in case of errors.  The configuration line is assumed to be
-   colon separated with these fields:
-
-   1. field: Hostname
-   2. field: Portnumber
-   3. field: Username
-   4. field: Password
-   5. field: Base DN
-
-   FILENAME and LINENO are used for diagnostic purposes only.
-*/
 ldap_server_t ldapserver_parse_one (char *line,
 				    const char *filename, unsigned int lineno);
 
