@@ -924,7 +924,14 @@ proc_wkd_get (ctrl_t ctrl, assuan_context_t ctx, char *line)
 
       err = get_dns_srv (ctrl, domain, "openpgpkey", NULL, &srvs, &srvscount);
       if (err)
-        goto leave;
+        {
+          /* Ignore server failed becuase there are too many resolvers
+           * which do not work as expected.  */
+          if (gpg_err_code (err) == GPG_ERR_SERVER_FAILED)
+            err = 0; /*(srvcount is guaranteed to be 0)*/
+          else
+            goto leave;
+        }
 
       /* Check for rogue DNS names.  */
       for (i = 0; i < srvscount; i++)
@@ -1005,7 +1012,7 @@ proc_wkd_get (ctrl_t ctrl, assuan_context_t ctx, char *line)
     {
       char *escapedmbox;
 
-      escapedmbox = http_escape_string (mbox, "%;?&=");
+      escapedmbox = http_escape_string (mbox, "%;?&=+#");
       if (escapedmbox)
         {
           uri = strconcat ("https://",
@@ -2142,18 +2149,18 @@ make_keyserver_item (const char *uri, uri_item_t *r_item)
   else if (!strcmp (uri, "https://keys.gnupg.net"))
     uri = "hkps://keyserver.ubuntu.com";
   else if (!strcmp (uri, "hkp://keys.gnupg.net"))
-    uri = "hkp://pgp.surf.nl";
+    uri = "hkp://keyserver.ubuntu.com";
   else if (!strcmp (uri, "http://keys.gnupg.net"))
-    uri = "hkp://pgp.surf.nl:80";
+    uri = "hkp://keyserver.ubuntu.com:80";
   else if (!strcmp (uri, "hkps://http-keys.gnupg.net")
            || !strcmp (uri, "http-keys.gnupg.net"))
     uri = "hkps://keyserver.ubuntu.com";
   else if (!strcmp (uri, "https://http-keys.gnupg.net"))
     uri = "hkps://keyserver.ubuntu.com";
   else if (!strcmp (uri, "hkp://http-keys.gnupg.net"))
-    uri = "hkp://pgp.surf.nl";
+    uri = "hkp://keyserver.ubuntu.com";
   else if (!strcmp (uri, "http://http-keys.gnupg.net"))
-    uri = "hkp://pgp.surf.nl:80";
+    uri = "hkp://keyserver.ubuntu.com:80";
 
   item = xtrymalloc (sizeof *item + strlen (uri));
   if (!item)
@@ -2276,7 +2283,7 @@ ensure_keyserver (ctrl_t ctrl)
     {
       /* If there is just one onion and one plain keyserver given, we take
          only one depending on whether Tor is running or not.  */
-      if (is_tor_running (ctrl))
+      if (!dirmngr_never_use_tor_p () && is_tor_running (ctrl))
         {
           ctrl->server_local->keyservers = onion_items;
           onion_items = NULL;
@@ -2287,7 +2294,7 @@ ensure_keyserver (ctrl_t ctrl)
           plain_items = NULL;
         }
     }
-  else if (!is_tor_running (ctrl))
+  else if (dirmngr_never_use_tor_p () || !is_tor_running (ctrl))
     {
       /* Tor is not running.  It does not make sense to add Onion
          addresses.  */
