@@ -120,7 +120,7 @@
 
 #if MY_GCC_VERSION >= 20500
 # define ATTR_PRINTF(f, a) __attribute__ ((format(printf,f,a)))
-# define ATTR_NR_PRINTF(f, a) __attribute__ ((__noreturn__, format(printf,f,a)))
+# define ATTR_NR_PRINTF(f, a) __attribute__ ((noreturn, format(printf,f,a)))
 #else
 # define ATTR_PRINTF(f, a)
 # define ATTR_NR_PRINTF(f, a)
@@ -1583,13 +1583,6 @@ main (int argc, char **argv)
           opt_store = 1;
           argc--; argv++;
         }
-      else if (!strcmp (*argv, "--html") || !strcmp (*argv, "--gnupgorg"))
-        {
-          /* These are dummy options to keep the Makefile simple.  It
-           * is not expected that the HTML files for upload are
-           * created with this included version of yat2m.  */
-          argc--; argv++;
-        }
       else if (!strcmp (*argv, "--select"))
         {
           argc--; argv++;
