@@ -106,7 +106,7 @@ gpg_keyboxd_deinit_session_data (ctrl_t ctrl)
               err = assuan_transact (kbl->ctx, "TRANSACTION commit",
                                      NULL, NULL, NULL, NULL, NULL, NULL);
               if (err)
-                log_error ("error committing last transaction: %s\n",
+                log_error ("error commiting last transaction: %s\n",
                             gpg_strerror (err));
               in_transaction = 0;
             }
@@ -223,9 +223,7 @@ open_context (ctrl_t ctrl, keyboxd_local_t *r_kbl)
           return err;
         }
 
-      /* We use D-lines in 2.4 for communication due to a bug with fd
-       * passing.  See T6512.  */
-      err = kbx_client_data_new (&kbl->kcd, kbl->ctx, 1 /*=use D-lines*/);
+      err = kbx_client_data_new (&kbl->kcd, kbl->ctx, 1);
       if (err)
         {
           assuan_release (kbl->ctx);
@@ -387,23 +385,6 @@ keydb_get_keyblock (KEYDB_HANDLE hd, kbnode_t *ret_kb)
 }
 
 
-/* Default status callback used to show diagnostics from the keyboxd  */
-static gpg_error_t
-keydb_default_status_cb (void *opaque, const char *line)
-{
-  const char *s;
-
-  (void)opaque;
-
-  if ((s = has_leading_keyword (line, "NOTE")))
-    log_info (_("Note: %s\n"), s);
-  else if ((s = has_leading_keyword (line, "WARNING")))
-    log_info (_("WARNING: %s\n"), s);
-
-  return 0;
-}
-
-
 
 /* Communication object for STORE commands.  */
 struct store_parm_s
@@ -483,8 +464,7 @@ keydb_update_keyblock (ctrl_t ctrl, KEYDB_HANDLE hd, kbnode_t kb)
   err = assuan_transact (hd->kbl->ctx, "STORE --update",
                          NULL, NULL,
                          store_inq_cb, &parm,
-                         keydb_default_status_cb, hd);
-
+                         NULL, NULL);
 
  leave:
   iobuf_close (iobuf);
@@ -535,7 +515,7 @@ keydb_insert_keyblock (KEYDB_HANDLE hd, kbnode_t kb)
   err = assuan_transact (hd->kbl->ctx, "STORE --insert",
                          NULL, NULL,
                          store_inq_cb, &parm,
-                         keydb_default_status_cb, hd);
+                         NULL, NULL);
 
  leave:
   iobuf_close (iobuf);
@@ -581,7 +561,7 @@ keydb_delete_keyblock (KEYDB_HANDLE hd)
   err = assuan_transact (hd->kbl->ctx, line,
                          NULL, NULL,
                          NULL, NULL,
-                         keydb_default_status_cb, hd);
+                         NULL, NULL);
 
  leave:
   return err;
@@ -668,8 +648,6 @@ search_status_cb (void *opaque, const char *line)
             }
         }
     }
-  else
-    err = keydb_default_status_cb (opaque, line);
 
   return err;
 }
