@@ -1294,6 +1294,8 @@ sign_file (ctrl_t ctrl, strlist_t filenames, int detached, strlist_t locusr,
   /* Setup the inner packet. */
   if (detached)
     {
+      size_t iobuf_size = iobuf_set_buffer_size(0) * 1024;
+
       if (multifile)
         {
           strlist_t sl;
@@ -1328,7 +1330,7 @@ sign_file (ctrl_t ctrl, strlist_t filenames, int detached, strlist_t locusr,
                   iobuf_push_filter (inp, text_filter, &tfx);
                 }
               iobuf_push_filter (inp, md_filter, &mfx);
-              while (iobuf_get (inp) != -1)
+              while (iobuf_read (inp, NULL, iobuf_size) != -1)
                 ;
               iobuf_close (inp);
               inp = NULL;
@@ -1339,7 +1341,7 @@ sign_file (ctrl_t ctrl, strlist_t filenames, int detached, strlist_t locusr,
       else
         {
           /* Read, so that the filter can calculate the digest. */
-          while (iobuf_get(inp) != -1)
+          while (iobuf_read (inp, NULL, iobuf_size) != -1)
             ;
 	}
     }
@@ -1658,8 +1660,9 @@ sign_symencrypt_file (ctrl_t ctrl, const char *fname, strlist_t locusr)
   {
     PKT_symkey_enc *enc = xmalloc_clear( sizeof *enc );
 
-    enc->version = 4;
+    enc->version = cfx.dek->use_aead ? 5 : 4;
     enc->cipher_algo = cfx.dek->algo;
+    enc->aead_algo = cfx.dek->use_aead;
     enc->s2k = *s2k;
     pkt.pkttype = PKT_SYMKEY_ENC;
     pkt.pkt.symkey_enc = enc;
@@ -1925,6 +1928,15 @@ update_keysig_packet (ctrl_t ctrl,
   /* Create a new signature packet.  */
   sig = copy_signature (NULL, orig_sig);
 
+  /* Don't generate version 3 signature, but newer.  */
+  if (sig->version == 3)
+    {
+      if (pk->version > 3)
+        sig->version = pk->version;
+      else
+        sig->version = 4;
+    }
+
   sig->digest_algo = digest_algo;
 
   /* We need to create a new timestamp so that new sig expiration
