@@ -447,7 +447,7 @@ app_send_devinfo (ctrl_t ctrl, int keep_looping)
         }
       send_status_direct (ctrl, "DEVINFO_END", "");
 
-      if (no_device && !keep_looping)
+      if (keep_looping == 0)
         break;
 
       card_list_wait ();
@@ -2053,7 +2053,7 @@ app_auth (card_t card, ctrl_t ctrl, const char *keyidstr,
 {
   gpg_error_t err;
 
-  if (!indata || !indatalen || !outdata || !outdatalen || !pincb)
+  if (!outdata || !outdatalen || !pincb)
     return gpg_error (GPG_ERR_INV_VALUE);
 
   if ((err = maybe_switch_app (ctrl, card, keyidstr)))
@@ -2062,6 +2062,10 @@ app_auth (card_t card, ctrl_t ctrl, const char *keyidstr,
     err = gpg_error (GPG_ERR_UNSUPPORTED_OPERATION);
   else
     {
+      if (card->app->apptype != APPTYPE_OPENPGP
+          && (!indata || !indatalen))
+        return gpg_error (GPG_ERR_INV_VALUE);
+
       if (DBG_APP)
         log_debug ("slot %d app %s: calling auth(%s)\n",
                    card->slot, xstrapptype (card->app), keyidstr);
