@@ -538,13 +538,19 @@ cmd_learn (assuan_context_t ctx, char *line)
   int only_keypairinfo = has_option (line, "--keypairinfo");
   int opt_multi = has_option (line, "--multi");
   int opt_reread = has_option (line, "--reread");
+  int opt_force = has_option (line, "--force");
   unsigned int flags;
   card_t card;
+  const char *keygrip = NULL;
 
   if ((rc = open_card (ctrl)))
     return rc;
 
-  card = card_get (ctrl, NULL);
+  line = skip_options (line);
+  if (strlen (line) == 40)
+    keygrip = line;
+
+  card = card_get (ctrl, keygrip);
   if (!card)
     return gpg_error (GPG_ERR_CARD_NOT_PRESENT);
 
@@ -581,7 +587,7 @@ cmd_learn (assuan_context_t ctx, char *line)
           return out_of_core ();
         }
 
-      if (!has_option (line, "--force"))
+      if (!opt_force)
         {
           char *command;
 
@@ -639,6 +645,7 @@ cmd_readcert (assuan_context_t ctx, char *line)
   unsigned char *cert;
   size_t ncert;
   card_t card;
+  const char *keygrip = NULL;
 
   if ((rc = open_card (ctrl)))
     return rc;
@@ -647,7 +654,10 @@ cmd_readcert (assuan_context_t ctx, char *line)
   if (!line)
     return gpg_error_from_syserror ();
 
-  card = card_get (ctrl, NULL);
+  if (strlen (line) == 40)
+    keygrip = line;
+
+  card = card_get (ctrl, keygrip);
   if (!card)
     {
       xfree (line);
@@ -974,7 +984,9 @@ pin_cb (void *opaque, const char *info, char **retstr)
 
   /* Fixme: Write an inquire function which returns the result in
      secure memory and check all further handling of the PIN. */
+  assuan_begin_confidential (ctx);
   rc = assuan_inquire (ctx, command, &value, &valuelen, MAXLEN_PIN);
+  assuan_end_confidential (ctx);
   xfree (command);
   if (rc)
     return rc;
@@ -2229,7 +2241,7 @@ static const char hlp_keyinfo[] =
   "Unless --data is given, the\n"
   "information is returned as a status line using the format:\n"
   "\n"
-  "  KEYINFO <keygrip> T <serialno> <idstr>\n"
+  "  KEYINFO <keygrip> T <serialno> <idstr> <usage>\n"
   "\n"
   "KEYGRIP is the keygrip.\n"
   "\n"
@@ -2237,9 +2249,13 @@ static const char hlp_keyinfo[] =
   "         smartcard.  If the serial number is not known a single\n"
   "         dash '-' is used instead.\n"
   "\n"
-  "IDSTR is the IDSTR used to distinguish keys on a smartcard.  If it\n"
+  "IDSTR is a string used to distinguish keys on a smartcard.  If it\n"
   "      is not known a dash is used instead.\n"
   "\n"
+  "USAGE is a string of capabilities of the key, 's' for sign, \n"
+  "'e' for encryption, 'a' for auth, and 'c' for cert.  If it is not\n"
+  "known a dash is used instead.\n"
+  "\n"
   "More information may be added in the future.";
 static gpg_error_t
 cmd_keyinfo (assuan_context_t ctx, char *line)
@@ -2284,14 +2300,15 @@ cmd_keyinfo (assuan_context_t ctx, char *line)
  * line.  */
 void
 send_keyinfo (ctrl_t ctrl, int data, const char *keygrip_str,
-              const char *serialno, const char *idstr)
+              const char *serialno, const char *idstr, const char *usage)
 {
   char *string;
   assuan_context_t ctx = ctrl->server_local->assuan_ctx;
 
-  string = xtryasprintf ("%s T %s %s%s", keygrip_str,
+  string = xtryasprintf ("%s T %s %s %s%s", keygrip_str,
                          serialno? serialno : "-",
                          idstr? idstr : "-",
+                         usage? usage : "-",
                          data? "\n" : "");
 
   if (!string)
