@@ -192,6 +192,9 @@
 (define (in-srcdir . names)
   (canonical-path (apply path-join (cons (getenv "abs_top_srcdir") names))))
 
+(define (in-objdir . names)
+  (canonical-path (apply path-join (cons (getenv "objdir") names))))
+
 ;; Split a list of paths.
 (define (pathsep-split s)
   (string-split s *pathsep*))
@@ -288,7 +291,7 @@
 ;; Otherwise this works like mkdtemp.
 (define (mkdtemp-autoremove . components)
   (let ((dir (apply mkdtemp components)))
-    (atexit (lambda () (unlink-recursively dir)))
+    (atexit (lambda () (if (= *exit-status* 0) (unlink-recursively dir))))
     dir))
 
 (define-macro (with-temporary-working-directory . expressions)
@@ -670,14 +673,14 @@
 	name))
 
   (package
-   (define (scm setup variant name path . args)
+   (define (scm setup name path . args)
      ;; Start the process.
      (define (spawn-scm args' in out err)
        (spawn-process-fd `(,*argv0* ,@(verbosity (*verbose*))
 				    ,(locate-test (test-name path))
 				    ,@(if setup (force setup) '())
 				    ,@args' ,@args) in out err))
-     (new variant name #f spawn-scm #f #f CLOSED_FD (expect-failure? name)))
+     (new name #f spawn-scm #f #f CLOSED_FD (expect-failure? name)))
 
    (define (binary setup name path . args)
      ;; Start the process.
@@ -685,9 +688,9 @@
        (spawn-process-fd `(,(test-name path)
 			   ,@(if setup (force setup) '()) ,@args' ,@args)
 			 in out err))
-     (new #f name #f spawn-binary #f #f CLOSED_FD (expect-failure? name)))
+     (new name #f spawn-binary #f #f CLOSED_FD (expect-failure? name)))
 
-   (define (new variant name directory spawn pid retcode logfd expect-failure)
+   (define (new name directory spawn pid retcode logfd expect-failure)
      (package
 
       ;; XXX: OO glue.
@@ -716,11 +719,7 @@
 
       (define (open-log-file)
 	(unless log-file-name
-	  (set! log-file-name (path-join
-			       (getenv "objdir")
-			       (if variant
-				   (string-append name "." variant ".log")
-				   (string-append name ".log")))))
+		(set! log-file-name (string-append (basename name) ".log")))
 	(catch '() (unlink log-file-name))
 	(open log-file-name (logior O_RDWR O_BINARY O_CREAT) #o600))
 
@@ -769,10 +768,7 @@
 		(seek logfd 0 SEEK_SET)
 		(splice logfd STDERR_FILENO)
 		(close logfd))
-	(echo (string-append (status-string) ":")
-	      (if variant
-		  (string-append "<" variant ">" name)
-		  name)))
+	(echo (string-append (status-string) ":") name))
 
       (define (xml)
 	(xx::tag
